/*
 *   Copyright 2000-2001 University of Illinois Board of Trustees
 *   Copyright 2000-2001 Mark D. Roth
 *   All rights reserved. 
 *
 *   pam_uid.c - PAM module for authentication based on uids
 *
 *   Mark D. Roth <roth@uiuc.edu>
 *   Computing and Communications Services Office
 *   University of Illinois at Urbana-Champaign
 */

#include <stdio.h>
#include <sys/types.h>
#include <syslog.h>
#include <pwd.h>

#include <security/pam_appl.h>
#include <security/pam_modules.h>

#ifndef LOG_AUTHPRIV
# define LOG_AUTHPRIV LOG_AUTH
#endif


int
pam_sm_authenticate(pam_handle_t *pamh, int flags,
		    int argc, const char *argv[])
{
	char *user, *cp;
	int i;
	uid_t uid;
	struct passwd *pwd;

#ifdef DEBUG
	printf("==> pam_sm_authenticate(pamh=0x%lx, flags=%d)\n", pamh, flags);
#endif

	if ((argc % 2) != 0)
		return PAM_AUTHINFO_UNAVAIL;

	if (pam_get_user(pamh, (char **)&user, "login: ") != PAM_SUCCESS)
	{
		syslog(LOG_AUTHPRIV | LOG_ERR,
		       "pam_uidfilter_authenticate: pam_get_user failed");
		return PAM_USER_UNKNOWN;
	}

#ifdef DEBUG
	printf("    pam_sm_authenticate(): user=\"%s\"\n", user);
#endif

	pwd = getpwnam(user);
	if (pwd == NULL)
	{
		syslog(LOG_AUTHPRIV | LOG_ERR,
		       "pam_uidfilter_authenticate: getpwnam(): %m");
		return PAM_USER_UNKNOWN;
	}

	for (i = 0; i < argc; i += 2)
	{
#ifdef DEBUG
		printf("    pam_sm_authenticate(): checking condition: %s %s\n",
		       argv[i], argv[i + 1]);
#endif

		uid = (uid_t)strtoul(argv[i + 1], &cp, 0);

		if ((strcmp(argv[i], "-ge") == 0
		     && pwd->pw_uid < uid)
		    || (strcmp(argv[i], "-gt") == 0
			&& pwd->pw_uid <= uid)
		    || (strcmp(argv[i], "-le") == 0
			&& pwd->pw_uid > uid)
		    || (strcmp(argv[i], "-lt") == 0
			&& pwd->pw_uid >= uid)
		    || (strcmp(argv[i], "-eq") == 0
			&& pwd->pw_uid != uid)
		    || (strcmp(argv[i], "-ne") == 0
			&& pwd->pw_uid == uid))
		{
			return PAM_AUTH_ERR;
		}
	}

	return PAM_SUCCESS;
}


int
pam_sm_setcred(pam_handle_t *pamh, int flags, int argc, const char *argv[])
{
#ifdef DEBUG
	syslog(LOG_AUTHPRIV | LOG_ERR,
	       "pam_uidfilter_setcred: not implemented");
#endif
	return PAM_SUCCESS;
}


int
pam_sm_acct_mgmt(pam_handle_t *pamh, int flags, int argc, const char *argv[])
{
	syslog(LOG_AUTHPRIV | LOG_ERR,
	       "pam_uidfilter_acct_mgmt: not implemented");
	return PAM_IGNORE;
}


int
pam_sm_open_session(pam_handle_t *pamh, int flags,
		    int argc, const char *argv[])
{
	syslog(LOG_AUTHPRIV | LOG_ERR,
	       "pam_uidfilter_open_session: not implemented");
	return PAM_IGNORE;
}


int
pam_sm_close_session(pam_handle_t *pamh, int flags,
		     int argc, const char *argv[])
{
	syslog(LOG_AUTHPRIV | LOG_ERR,
	       "pam_uidfilter_close_session: not implemented");
	return PAM_IGNORE;
}


int
pam_sm_chauthtok(pam_handle_t *pamh, int flags, int argc, const char *argv[])
{
	syslog(LOG_AUTHPRIV | LOG_ERR,
	       "pam_uidfilter_chauthtok: not implemented");
	return PAM_IGNORE;
}


